from trac.wiki.macros import WikiMacroBase
from trac.resource import Resource, get_resource_url
from bzrlib import bzrdir
from trac.util.datefmt import format_datetime,pretty_timedelta
from genshi.template import TemplateLoader
from trac.web.chrome import Chrome

__all__ = ['BzrTagListMacro']

class BzrTagListMacro(WikiMacroBase):
    '''This macro support bzr tags, depend on trac-bzr.

        Version : alpha

        Author  : buaa.byl@gmail.com

        Usage example:
        {{{
        [[BzrTagList]]
        }}}

        Render result:
        ||Changeset ||Tag name    ||
        ||[1]       ||sample-0.0.1||
        ||[1]       ||sample-0.0.2||
        ||[2]       ||sample-0.0.3||

        I am not clear of bzrlib and trac, 
        so I just read 'bzrlib/tests/test_bzrdir.py', 
        'trac/config.py', 'trac/wiki/macros.py',
        'tracbzr/backend.py' and try to implement this macro.
    '''
    def get_macros(self):
        '''Macro name is 'BzrTagList' '''
        name = self.__class__.__name__
        if name.endswith('Macro'):
            name = name[:-5]
        yield name

    def expand_macro(self, formatter, name, content):
        #get repository type/dir from trac.ini
        repository_type = self.env.config.get('trac','repository_type')
        if repository_type == 'bzr':
            repository_dir = self.env.config.get('trac','repository_dir')
        else:
            repository_dir = ''

        #if support bzr repository_type and repository_dir is set,
        #I will try to load bzr repo, and get tags information.
        if repository_dir:
            #get bzr branch
            #TODO:may be better to open_from_transport, but I am not clear.
            target_bzrdir = bzrdir.BzrDir.open(repository_dir)
            branch = target_bzrdir.open_branch()

            #get tags, result is dictory, and key is tag's name string
            #vaule is revision, need change to revno.
            tagrevision_dict = branch.tags.get_tag_dict()

            #sorted tags with revno, and save all tag name into list
            tagrevno_dict = {}
            for tagname in tagrevision_dict:
                revno = branch.revision_id_to_dotted_revno(tagrevision_dict[tagname])
                timestamp = branch.repository.get_revision(tagrevision_dict[tagname]).timestamp
                committer = branch.repository.get_revision(tagrevision_dict[tagname]).committer
                message   = branch.repository.get_revision(tagrevision_dict[tagname]).message
                if revno in tagrevno_dict:
                    tagrevno_dict[revno] = tagrevno_dict[revno] + \
                            [{'tagname':tagname, 'timestamp':timestamp, \
                              'committer':committer, 'message':message}]
                else:
                    tagrevno_dict[revno] = \
                            [{'tagname':tagname, 'timestamp':timestamp, \
                              'committer':committer, 'message':message}]

            #get base url of browser
            #TODO:If there are better way to get url
            resource = Resource('browser', '/')
            browser_url = get_resource_url(self.env, resource, formatter.context.href)

            #resource = Resource('changeset', '/')
            #changeset_url = get_resource_url(self.env, resource, formatter.context.href)
            changeset_url = browser_url.replace('browser','changeset')
            
            #Get all revno with tag infomation
            taglist = [ ]
            id = 0

            for revno in sorted(tagrevno_dict):
                tagitem = {}
                if len(revno) == 1:
                    revno_str = "%d" % revno[0]
                elif len(revno) == 2:
                    revno_str = "%d.%d" % (revno[0], revno[1])
                elif len(revno) == 3:
                    revno_str = "%d.%d.%d" % (revno[0], revno[1], revno[2])
                else:
                    revno_str = ''
                
                #Save revno
                tagitem['revno'] = revno_str

                #Get all tag information
                for taginfo in sorted(tagrevno_dict[revno]):
                    tagitem['tagname']   = taginfo['tagname']
                    tagitem['timestamp'] = format_datetime(taginfo['timestamp'])
                    tagitem['timedelta'] = pretty_timedelta(taginfo['timestamp'])
                    tagitem['committer'] = taginfo['committer']
                    tagitem['message'] = taginfo['message']
                    if id % 2 == 0:
                        tagitem['id'] = 'odd'
                    else:
                        tagitem['id'] = 'even'

                    id = id + 1
                    taglist.append(tagitem)

            dict = {'changeset_url':changeset_url, 'browser_url':browser_url, 'taglist': taglist}
            
            #render template
            tmpl = Chrome(self.env).load_template('bzrtagstmpl.xhtml')
            stream = tmpl.generate(dict=dict, fragment=True)
            result = stream.render('html', doctype='html')

        else:
            result = ''

        return result

